// Copyright (c) 2013 by István Váradi

// This file is part of XPLRA, a remote-access plugin for X-Plane

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

#ifndef XPLRA_HOTKEYHANDLER_H
#define XPLRA_HOTKEYHANDLER_H
//------------------------------------------------------------------------------

#include <hu/varadiistvan/scpl/Mutex.h>

#include <XPLMDefs.h>

#include <list>

#include <inttypes.h>

//------------------------------------------------------------------------------

namespace hu { namespace varadiistvan { namespace scpl { namespace io {

class DataStream;

} } } }

//------------------------------------------------------------------------------

namespace xplra {

//------------------------------------------------------------------------------

/**
 * The handler for hotkeys.
 */
class HotkeyHandler
{
public:
    /**
     * A set of hotkeys for a client.
     */
    class Hotkeys
    {
    private:
        /**
         * The number of hotkeys.
         */
        size_t numHotkeys;

        /**
         * The hotkey codes.
         */
        uint16_t* hotkeys;

        /**
         * The array of indicators whether the hotkey at the given
         * index in the array of codes has been pressed.
         */
        uint8_t* pressed;

    public:
        /**
         * Construct the set of hotkeys by reading the data from the
         * given data stream.
         */
        Hotkeys(size_t numHotkeys, hu::varadiistvan::scpl::io::DataStream& stream);

        /**
         * Destroy the set of hotkeys.
         */
        ~Hotkeys();

        /**
         * Determine if this hotkey set is valid.
         */
        bool isValid() const;

        /**
         * Handle the hotkey with the given code. If it matches any of
         * the codes registered, the corresponding value in the
         * pressed array will be set atomically.
         *
         * @return whether the hotkey was handled or not.
         */
        bool handleKey(uint16_t code);

        /**
         * Write the pressed indicators into the given stream. It also
         * clears the indicators.
         */
        void writePressed(hu::varadiistvan::scpl::io::DataStream& stream);
    };

    friend class Hotkeys;

private:
    /**
     * Type for the the list of hotkey sets.
     */
    typedef std::list<Hotkeys*> hotkeySets_t;

    /**
     * The callback function receiving the keypresses.
     */
    static int keyCallback(char c, XPLMKeyFlags flags, char virtualKey,
                           void* refCon);

    /**
     * Mutex to protect some common data structures.
     */
    hu::varadiistvan::scpl::Mutex mutex;

    /**
     * The list of hotkey sets.
     */
    hotkeySets_t hotkeySets;

public:
    /**
     * Construct the hotkey handler.
     */
    HotkeyHandler();

    /**
     * Register the given set of hotkeys.
     */
    void registerHotkeys(Hotkeys* hotkeys);

    /**
     * Unregister the given set of hotkeys.
     */
    void unregisterHotkeys(Hotkeys* hotkeys);

protected:
    /**
     * Handle the given hotkey code.
     *
     * @return true if the hotkey was handled, false otherwise.
     */
    bool handleKey(char c, XPLMKeyFlags flags, char virtualKey);
};

//------------------------------------------------------------------------------
// Inline definitions
//------------------------------------------------------------------------------

inline bool HotkeyHandler::Hotkeys::isValid() const
{
    return hotkeys!=0;
}

//------------------------------------------------------------------------------

} /* namespace xplra */

//------------------------------------------------------------------------------
#endif // XPLRA_HOTKEYHANDLER_H

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
